# HTTP

## Introduzione

`HTTP` sta per *Hypertext Transfer Protocol* ed è il protocollo incaricato di gestire lo scambio di richieste e risposte http in un'architettura client-server. Questo protocollo è utilizzato per lo scambio di pagine web e altri contenuti multimediali. HTTP è associato alla well-known port *80*, mentre `HTTPS` (la versione sicura di HTTP) utilizza la porta *443*. Ciò significa che il processo sul server relativo a questi servizi è tipicamente legato a queste porte.

In quanto protocollo dell'application layer, HTTP si appoggia al TCP del transport layer per garantire una comunicazione affidabile e robusta. Prima che avvenga lo scambio dei veri e propri messaggi, è necessario stabilire una connessione TCP tra il client e il server.

## Tipologie di connessione HTTP

- **Connessione non persistente:** un metodo tradizionale in cui per ogni richiesta HTTP viene instaurata una sessione TCP separata. Questo approccio, essendo meno efficiente, può comportare un uso eccessivo di risorse, in particolare quando vengono richieste numerose risorse in rapida successione
- **Connessione persistente:** con questo metodo, una volta stabilita la connessione TCP tra il client e il server, tutte le risorse vengono scambiate utilizzando la stessa connessione. Solo al termine dello scambio della totalità delle risorse la connessione viene chiusa. Questo approccio è molto più efficiente, poiché riduce il sovraccarico derivante dall'instaurazione ripetuta di nuove connessioni TCP

## HTTP Request

Una richiesta HTTP è strutturata in quattro macrosezioni:

1. **Riga di richiesta**: contiene il metodo HTTP (ad esempio, GET, POST, PUT, DELETE, ecc.) e la versione di HTTP utilizzata, solitamente la versione 2
2. **Intestazione** o **Header**: ena serie di coppie chiave-valore che forniscono al server ulteriori informazioni, come ad esempio il tipo di contenuto, le informazioni sul client, le preferenze sulla codifica, e altro ancora
3. **Riga vuota**: Separa le intestazioni dal corpo della richiesta
4. **Corpo della richiesta** o **Body** (Opzionale): contiene i dati da inviare al server

![http-request](asset/img/http-request.png)

### Intestazione della Richiesta HTTP

Gli header di una richiesta HTTP vengono utilizzati dal client per fornire al server informazioni utili all'elaborazione della risposta. Alcuni dei principali campi utilizzati:

- `User-Agent`: specifica il tipo di client utilizzato per la richiesta, ad esempio il browser o l'applicazione
- `Accept`: indica quali tipi di contenuti il client è in grado di gestire. Ad esempio, l'header `Accept-Language` comunica le preferenze linguistiche del browser, permettendo al server di restituire contenuti nella lingua preferita
- `Connection`: Determina se la connessione TCP deve rimanere aperta o essere chiusa subito dopo la risposta. Il valore `keep-alive` serve a mantenere attiva la connessione
- `Host`: specifica l'host e la porta di destinazione della richiesta
- `Referrer`: indica il percorso o path da cui è partita la richiesta
- `Cookie`: invio dei cookie associati alla sessione o ad altre informazioni di identificazione, utili per mantenere lo stato della sessione utente e per personalizzare l'esperienza
- `Content-Type`: Specifica il tipo di contenuto che il client sta inviando nel corpo della richiesta, ad esempio `application/json` o `text/html`
- `Content-Length`: indica la dimensione, in byte, del contenuto presente nel corpo della richiesta
- `If-Modified-Since`: questo campo contiene un timestamp che indica l'ultima modifica nota al client. Il server utilizza questo dato per decidere se inviare il contenuto richiesto o restituire un codice 304 (*Not Modified*) se il contenuto non è stato aggiornato successivamente alla data specificata

![header-http](asset/img/header-http.png)

### Metodi HTTP

I metodi HTTP indicano al server l'azione da eseguire su una risorsa. I principali metodi:

- `GET`: il metodo più comune, serve per richiedere una risorsa presente sul server. Non utilizza il corpo (body) della richiesta; eventuali parametri vengono passati direttamente nell'URL, dopo il carattere `?` e separati tramite il carattere `&`. Ad esempio: `https://miosito.com/?filtro=1&rilasciato=false`
- `POST`: utilizzato per inviare dati al server, ad esempio i dati di un modulo di login. In questo caso, le informazioni vengono trasmesse tramite il corpo della richiesta
- `HEAD`: serve per richiedere informazioni (metadata) su una risorsa, senza scaricarne l'intero contenuto. Questo metodo è utile per verificare ad esempio l'esistenza della risorsa
- `PUT`: utilizzato per creare o aggiornare una risorsa sul server. Questo metodo, tuttavia, è spesso disabilitato per motivi di sicurezza
- `TRACE`: impiegato a fini diagnostici e di debugging, aiutando a identificare eventuali problemi lungo il percorso della richiesta
- `DELETE`: consente di eliminare una risorsa sul server. Anche questo metodo, per ragioni di sicurezza, è spesso disabilitato di default
- `OPTIONS`: utilizzato per verificare quali metodi HTTP sono supportati dal server, consentendo così al client di conoscere le possibili azioni disponibili

## URI, URL e URN

Un `URI` (*Uniform Resource Identifier*) è un meccanismo per identificare in modo univoco una risorsa generica. Esistono due specializzazioni principali:

- `URL` (*Uniform Resource Locator*): è comunemente utilizzato nel web e specifica il percorso dove la risorsa può essere raggiunta. Segue il formato: `protocollo://{hostname o nome dominio/ip}[:porta]/percorso/?parametro1=valore1&parametro2=valore2...`
- `URN` (*Uniform Resource Name*): rappresenta il contesto o namespace in cui è inserita la risorsa, senza indicarne il percorso di accesso. Un URN consente, ad esempio, di classificare un libro in base al suo genere, indipendentemente dalla sua ubicazione specifica
